/**
* @file tests.cpp
*      
*/

#include "tests.h"

UINT16 FCS16(UINT16 fcs, PUINT8 pbuf, UINT32 buflen)
{
	while (buflen--)
		fcs = (fcs >> 8) ^ fcstab16[(fcs ^ *pbuf++) & 0xFF];
	return ~fcs;
}

UINT8 FCS8(PUINT8 pbuf, UINT32 buflen)
{
	UINT32 i;
	UINT8 fcs8 = 0;

	for (i = 0; i < buflen; i++)
		fcs8 = pbHECSyndromeTable[fcs8 ^ pbuf[i]];
	return fcs8 ^ COSET_LEADER;
}

/**    */
UINT32 check_random_data(UINT8* pbuf, UINT32 buflen, UINT32 sid, UINT32 eid)
{
	UINT32 bits, bit;

	if (eid == (UINT32)-1)
	{
		bits = 0;
		for (bit = 0; bit < 8; bit++)
			if ((sid & (1 << bit)) != 0)
				bits++;
		if ((bits & 0x07) != ((sid >> 8) & 0x07))
			return -1;
	}
	else
	{
		bits = 0;
		for (bit = 0; bit < 11; bit++)
			if ((sid & (1 << bit)) != 0)
				bits++;
		if (eid != (((bits & 0x03) << 16) | FCS16(FCS16_INIT, (UINT8*)&sid, 2)))
			return -1;
	}

	if (buflen == 1)
		return (((pbuf[0] >> 4) ^ 0x0F) == (pbuf[0] & 0x0F)) ? 0 : -1;
	else if (buflen > 1)
		return (pbuf[buflen - 1] == FCS8(pbuf, buflen - 1)) ? 0 : -1;
	return -1;
}

UINT32 build_trigger_data(PUINT8 pbuf, UINT32 bufsize, PUINT32 psid, PUINT32 peid, UINT32 channel, UINT32 nbuf) {
	pbuf[0] = 0;
	*psid = channel;
	*peid = nbuf;
	return 1;
}

/**      */
UINT32 build_seeded_data(PUINT8 pbuf, UINT32 bufsize, PUINT32 psid, PUINT32 peid, UINT8 d0, UINT8 d1, UINT8 d2, UINT8 d3, UINT8 d4) {
	UINT8 val, bits, bit;
	UINT32 i;
	UINT32 buflen;

	*psid = 0;
	*peid = 0;

	if ((rand() % 2) == 0)
	{
		val = (UINT8)rand();
		bits = 0;
		for (bit = 0; bit < 8; bit++)
			if ((val & (1 << bit)) != 0)
				bits++;
		*psid = ((bits & 0x07) << 8) | val;
		*peid = (UINT32)-1;
	}
	else
	{
		*psid = rand() & 0x7FF;
		bits = 0;
		for (bit = 0; bit < 11; bit++)
			if ((*psid & (1 << bit)) != 0)
				bits++;
		*peid = ((bits & 0x03) << 16) | FCS16(FCS16_INIT, (PUINT8)psid, 2);
	}

	buflen = 1 + rand() % bufsize;
	buflen = (buflen < 6) ? 6 : buflen;

	if (d4 == 255) {
		buflen = 5;
	}

	pbuf[0] = d0;
	pbuf[1] = d1;
	pbuf[2] = d2;
	pbuf[3] = d3;
	pbuf[4] = d4;

	for (i = 5; i < buflen - 1; i++)
		pbuf[i] = (uint8_t)rand();

	pbuf[buflen - 1] = FCS8(pbuf, buflen - 1);

	return buflen;
}

/**      */
UINT32 build_random_data(PUINT8 pbuf, UINT32 bufsize, PUINT32 psid, PUINT32 peid) {
	UINT8 val, bits, bit;
	UINT32 i;
	UINT32 buflen;

	*psid = 0;
	*peid = 0;

	if ((rand() % 2) == 0)
	{
		val = (UINT8)rand();
		bits = 0;
		for (bit = 0; bit < 8; bit++)
			if ((val & (1 << bit)) != 0)
				bits++;
		*psid = ((bits & 0x07) << 8) | val;
		*peid = (UINT32)-1;
	}
	else
	{
		*psid = rand() & 0x7FF;
		bits = 0;
		for (bit = 0; bit < 11; bit++)
			if ((*psid & (1 << bit)) != 0)
				bits++;
		*peid = ((bits & 0x03) << 16) | FCS16(FCS16_INIT, (PUINT8)psid, 2);
	}

	buflen = 1 + rand() % bufsize;

	if (buflen == 1)
	{
		val = (UINT8)(rand() & 0x0F);
		pbuf[0] = ((val ^ 0x0F) << 4) | val;
	}
	else
	{
		for (i = 0; i < buflen - 1; i++) {
			pbuf[i] = (UINT8)rand();
		}
		
		pbuf[buflen - 1] = FCS8(pbuf, buflen - 1);
	}

	return buflen;
}

/**   */
UINT32 build_seq_data(PUINT8 pbuf, UINT32 bufsize, PUINT32 psid, PUINT32 peid) {
	UINT8 val, bits, bit;
	UINT32 i;
	UINT32 buflen;

	*psid = 0;
	*peid = 0;

	if ((rand() % 2) == 0)
	{
		val = (UINT8)rand();
		bits = 0;
		for (bit = 0; bit < 8; bit++)
			if ((val & (1 << bit)) != 0)
				bits++;
		*psid = ((bits & 0x07) << 8) | val;
		*peid = (UINT32)-1;
	}
	else
	{
		*psid = rand() & 0x7FF;
		bits = 0;
		for (bit = 0; bit < 11; bit++)
			if ((*psid & (1 << bit)) != 0)
				bits++;
		*peid = ((bits & 0x03) << 16) | FCS16(FCS16_INIT, (PUINT8)psid, 2);
	}

	buflen = 6;
	for (i = 0; i < buflen - 1; i++)
		pbuf[i] = i;
	pbuf[buflen - 1] = FCS8(pbuf, buflen - 1);


	return buflen;
}

/**    */
int tt_check_random_data(uint8_t* pbuf, uint32_t buflen, uint32_t sid, uint32_t eid)
{
	uint8_t bits, bit;

	if (eid == (uint32_t)-1)
	{
		bits = 0;
		for (bit = 0; bit < 8; bit++)
			if ((sid & (1 << bit)) != 0)
				bits++;
		if ((bits & 0x07) != ((sid >> 8) & 0x07)) {
			return -1;
		}
	}
	else
	{
		bits = 0;
		for (bit = 0; bit < 11; bit++)
			if ((sid & (1 << bit)) != 0)
				bits++;
		if (eid != (((bits & 0x03) << 16) | FCS16(FCS16_INIT, (uint8_t*)&sid, 2))){
			return -1;
		}
	}

	if (buflen == 1)
		return (((pbuf[0] >> 4) ^ 0x0F) == (pbuf[0] & 0x0F)) ? 0 : -1;
	else if (buflen > 1)
		return (pbuf[buflen - 1] == FCS8(pbuf, buflen - 1)) ? 0 : -1;
	return -1;
}


/**  ()   */
void nmcan_wait_tx(HANDLE* ttcan, int channel, int buf) {
	int txbnctrl[] = { 0x30, 0x40, 0x50 };
	UINT8 reg = 0;
	do {
		CAN_read_can(ttcan, txbnctrl[buf], channel, &reg, 1);
	} while ((reg & 0x8) != 0);
}

/**     */
void nmcan_wait_tx_sane(HANDLE* ttcan, int channel, int buf) {
	CAN_SEND_DATA_NOW sd;
	DWORD size, err;

	sd.channel = channel;
	sd.nBuf = buf;
	sd.timeout = 1000;

	if (!DeviceIoControl(ttcan, IOCTL_WAIT_FOR_TX, &sd, sizeof(CAN_SEND_DATA_NOW), NULL, 0, &size, NULL)) {
		err = GetLastError();
		printf("Error x%X waiting for tx\n", err);
	}

}

void check_errors(HANDLE* ttcan, int channel) {
	UINT32 reg;
	UINT8 rec, tec, eflg;
	CAN_read_bar(ttcan, gMirrorRegs[channel], &reg);

	tec =  (reg >> 16) & 0xFF;
	rec =  (reg >> 8) & 0xFF;
	eflg = (reg >> 0) & 0xFF;

	if (tec > gTEC) {
		gTecChanges++;
	}

	if (rec > gREC) {
		gRecChanges++;
	}

	if (eflg > gEFLG) {
		gEflgChanges++;
	}

	if (gTecChanges || gRecChanges || gEflgChanges) {
		printf("REC=%u; TEC=%u; EFLG=%u\n", gRecChanges, gTecChanges, gEflgChanges);
		logToFile("REC=%u; TEC=%u; EFLG=%u", gRecChanges, gTecChanges, gEflgChanges);
	}
	
	gREC = rec;
	gTEC = tec;
	gEFLG = eflg;
}

FILE* openLogFile(const char* identifier) {
	time_t rawtime;
	struct tm timeinfo;
	char tbuf[40], fbuf[80];
	FILE* file = NULL;

	time(&rawtime);
	localtime_s(&timeinfo, &rawtime);

	strftime(tbuf, 40, "%F_%H_%M_%S", &timeinfo);
	sprintf_s(fbuf, "%s_%s_%u.log", tbuf, identifier, GetCurrentProcessId());

	fopen_s(&file, fbuf, "w");
	if (file != NULL) {
		printf("PID is %u\nLogging to %s\n", GetCurrentProcessId(), fbuf);
		gLog = TRUE;
	}
	else {
		printf("Error logging to %s, can't open file\n", fbuf);
	}

	gLogFile = file;
	return file;
}

void logToFile(const char* format, ...) {
	if (!gLog) {
		return;
	}

	time_t rawtime;
	struct tm timeinfo;
	char tbuf[40];

	time(&rawtime);
	localtime_s(&timeinfo, &rawtime);


	strftime(tbuf, 40, "%F %T", &timeinfo);
	fprintf(gLogFile, "%s: ", tbuf);

	va_list ptr;
	va_start(ptr, format);
	vfprintf(gLogFile, format, ptr);
	va_end(ptr);

	fprintf(gLogFile, "\n");
}