/**
* @file nmcantestgetstats.cpp
*     
*/

#include "..\misc\misc.h"
#include "..\misc\tests.h"


int main(int argc, char* argv[])
{
	DWORD err;
	CAN_device_list* devices;
	HANDLE can;
	int device_id, count;
	int channel, reset = 0;

	std::cout << "CAN Serial Controller device test v." << TEST_VERSION << std::endl << COPYRIGHT << std::endl << "Built on " << __TIMESTAMP__ << std::endl << std::endl;

	std::cout << "Displaying stats" << std::endl;

	if (argc < 3) {
		std::cout << "usage: " << argv[0] << " <device_id> <channel: (0|1)> [reset]" << std::endl;
		return 1;
	}

	//   
	timeBeginPeriod(1);

	device_id = strtol(argv[1], NULL, 10);

	devices = CAN_enumerate();
	count = devices->count;

	if (count == 0) {
		std::cout << "No CAN devices found" << std::endl;
		return 1;
	}

	if (device_id >= count) {
		std::cout << "Invalid zero-based device number, devices available: " << count << std::endl;
		return 1;
	}

	std::cout << "found " << count << " devices" << std::endl;
	printf("Using %ws\n", devices->device[device_id].device_name);

	can = CAN_open_device(devices->device + device_id);

	CAN_enumerate_free(devices);

	if (can == INVALID_HANDLE_VALUE) {
		std::cout << "Bad handle" << std::endl;
		return 1;
	}

	channel = atoi(argv[2]);

	if (channel != 0 && channel != 1) {
		std::cout << "Bad channel id" << std::endl;
		return 1;
	}

	printf("Using device %d, channel %d\n", device_id, channel);

	CAN_CANXSTAT stats;
	stats.channel = channel;

	if (argc > 3 && strcmp("reset", argv[3]) == 0) {
		reset = 1;
	}

	err = CAN_get_stats(&can, channel, reset, &stats);
	if (err != 0) {
		printf("error x%X gettings stats\n", err);
		return 1;
	}


	printf("CNT_TX\t\t%zu\n", stats.cnt_tx);
	printf("CNT_RX\t\t%zu\n", stats.cnt_rx);
	printf("CNT_ERR_TX\t%zu\n", stats.cnt_err_tx);
	printf("CNT_ERR_RX\t%zu\n", stats.cnt_err_rx);
	printf("CNT_ERR_OVF_TX\t%zu\n", stats.cnt_err_ovf_tx);
	printf("CNT_ERR_ABT\t%zu\n", stats.cnt_err_abt);
	printf("CNT_ERR_BOFF\t%zu\n", stats.cnt_err_boff);
	printf("CNT_ERR_EP\t%zu\n", stats.cnt_err_ep);
	printf("CNT_ERR_OVF_RX\t%zu\n", stats.cnt_err_ovf_rx);
	printf("CNT_ERR_WRN\t%zu\n", stats.cnt_err_wrn);


	return 0;
}