/**
* @file nmcantestinit.cpp
*  
*/

#include "..\misc\misc.h"
#include "..\misc\tests.h"

int main(int argc, char* argv[])
{
	DWORD err;
	CAN_device_list* devices;
	HANDLE can;
	int device_id, count;
	int channel, speed, mode;

	std::cout << "CAN Serial Controller device test v." << TEST_VERSION << std::endl << COPYRIGHT << std::endl << "Built on " << __TIMESTAMP__ << std::endl << std::endl;

	std::cout << "Device initialization" << std::endl;

	if (argc < 5) {
		std::cout << "usage: " << argv[0] << " <device_id> <channel: (0|1)> <speed: (125|250|500|1000)> <sendmode(0 - native, 1 - fifo)>" << std::endl;
		return 1;
	}

	//   
	timeBeginPeriod(1);

	device_id = strtol(argv[1], NULL, 10);

	devices = CAN_enumerate();
	count = devices->count;

	if (count == 0) {
		std::cout << "No CAN devices found" << std::endl;
		return 1;
	}

	if (device_id >= count) {
		std::cout << "Invalid zero-based device number, devices available: " << count << std::endl;
		return 1;
	}

	std::cout << "found " << count << " devices" << std::endl;
	printf("Using %ws\n", devices->device[device_id].device_name);

	can = CAN_open_device(devices->device + device_id);

	CAN_enumerate_free(devices);

	if (can == INVALID_HANDLE_VALUE) {
		std::cout << "Bad handle" << std::endl;
		return 1;
	}

	channel = atoi(argv[2]);
	speed = atoi(argv[3]);
	mode = atoi(argv[4]);

	if (channel != 0 && channel != 1) {
		std::cout << "Bad channel id" << std::endl;
		return 1;
	}

	printf("Using device %d, channel %d\n", device_id, channel);

	if (speed != 125 && speed != 250 && speed != 500 && speed != 1000) {
		std::cout << "Bad speed" << std::endl;
		return 1;
	}

	if (mode != 0 && mode != 1) {
		std::cout << "Bad mode" << std::endl;
		return 1;
	}

	CAN_DEVINFO devinfo;
	CAN_DRVINFO drvinfo;

	//   
	DeviceIoControl(can, IOCTL_DEVICE_VERSION, NULL, 0, &devinfo, sizeof(CAN_DEVINFO), &err, NULL);
	printf("CAN REV_%X\n", devinfo.revision_id);
	printf("Update mode: %u, Has FIFO: %u, CAN: %u\n", devinfo.isOnlyUpdate, devinfo.isFifoAvailable, devinfo.isTtcan);

	//   
	DeviceIoControl(can, IOCTL_DRIVER_VERSION, NULL, 0, &drvinfo, sizeof(CAN_DRVINFO), &err, NULL);
	printf("Driver v.%u.%u.%u built %u.%u.%u\n", (drvinfo.version >> 24), (drvinfo.version >> 16) & 0xFF, (drvinfo.version >> 8) & 0xff, (drvinfo.build_date >> 16) & 0xFFFF, (drvinfo.build_date >> 8) & 0xFF, drvinfo.build_date & 0xFF);

	//   

	// 
	CAN_reset_channel(&can, channel);
	// dma
	err = CAN_dma_enable(&can);
	if (err) {
		printf("WARN: dma enable returned x%X\n", err);
	}

	//  ,    ,   
	CAN_set_mode(&can, channel, CAN_CONF);
	CAN_set_speed(&can, channel, speed);
	CAN_set_timeouts(&can, channel, 4 * 4 * 16 * 8 * (1000 / speed), 4 * 16 * 8 * (1000 / speed));
	CAN_set_mode(&can, channel, CAN_WORK);

	//
	BAR_IOCTL_REG_REQ rg;
	rg.reg_addr = (channel == 0) ? 0x1040 : 0x1050;
	DeviceIoControl(can, IOCTL_READ_BAR, &rg, sizeof(BAR_IOCTL_REG_REQ), &rg, sizeof(BAR_IOCTL_REG_REQ), &err, NULL);
	rg.buf |= 0x7;
	DeviceIoControl(can, IOCTL_WRITE_BAR, &rg, sizeof(BAR_IOCTL_REG_REQ), NULL, 0, &err, NULL);

	UINT8 cbuf = 0xFF;
	CAN_write_can(&can, 0x2B, channel, &cbuf, 1);

	//    fifo   txpause  fifo  hpfifo
	CAN_set_send_mode(&can, channel, mode == 1 ? CAN_FIFO : CAN_NATIVE);
	CAN_set_tx_pause(&can, channel, 0, 0);
	CAN_set_tx_pause(&can, channel, 0, 1);

	return 0;
}