/**
* @file nmcantestrxfifo.cpp
*    FIFO, 2  ,    
*/

#include "..\misc\misc.h"
#include "..\misc\tests.h"

int main(int argc, char* argv[])
{
	CAN_device_list* devices;
	HANDLE can;
	int device_id, count;
	int channel, speed, mode;

	std::cout << "CAN Serial Controller device test v." << TEST_VERSION << std::endl << COPYRIGHT << std::endl << "Built on " << __TIMESTAMP__ << std::endl << std::endl;

	std::cout << "Testing RX (FIFO mode)" << std::endl;

	if (argc < 3) {
		std::cout << "usage: " << argv[0] << " <device_id> <channel: (0|1)> [dump]" << std::endl;
		return 1;
	}

	//setting timer precision, it will reset back to default after process finishes
	timeBeginPeriod(1);

	device_id = strtol(argv[1], NULL, 10);

	devices = CAN_enumerate();
	count = devices->count;

	if (count == 0) {
		std::cout << "No CAN devices found" << std::endl;
		return 1;
	}

	if (device_id >= count) {
		std::cout << "Invalid zero-based device number, devices available: " << count << std::endl;
		return 1;
	}

	std::cout << "found " << count << " devices" << std::endl;
	printf("Using %ws\n", devices->device[device_id].device_name);

	can = CAN_open_device(devices->device + device_id);

	CAN_enumerate_free(devices);

	if (can == INVALID_HANDLE_VALUE) {
		std::cout << "Bad handle" << std::endl;
		return 1;
	}

	channel = atoi(argv[2]);

	if (channel != 0 && channel != 1) {
		std::cout << "Bad channel id" << std::endl;
		return 1;
	}

	if (argc > 3 && strcmp("dump", argv[3]) == 0) {
		gLogFile = openLogFile("can_rxfifo");
	}

	printf("Using device %d, channel %d\n", device_id, channel);

	DMA_SLOT_CAN* dmareq = (DMA_SLOT_CAN*)malloc(sizeof(DMA_SLOT_CAN) * DMA_STR_MAXSIZE);
	DWORD dwStart = GetTickCount();
	int blocks, blockC = 0, bytesC = 0, bytesD = 0, blocksErr = 0;
	int mfifo = 0, mhpfifo = 0;
	UINT32 nbuf, tmr, ntu, div, sid, eid, flowerr = 0, flowerr2 = 0;
	UINT8 chan, datasize, ppktid, ppktid2;
	UINT8 data[15];
	int initial1 = 1, initial2 = 1;

	ppktid = 0;
	ppktid2 = 0;
	for (;;) {
		blocks = CAN_read_dma(&can, channel, DMA_STR_MAXSIZE, &dmareq, 0);
		blockC += blocks;
		for (int i = 0; i < blocks; i++) {

			CAN_decode_buf(dmareq + i, &nbuf, &chan, &tmr, &ntu, &div, &sid, &eid, data, &datasize);
			logToFile("rx fifo %3u s:x%3X e:x%8X l:%u hpf:%u", data[0], sid, eid, datasize, data[4]);
			if (tt_check_random_data(data, datasize, sid, eid) == -1) {
				blocksErr++;
				logToFile("block error");
			}
			else {
				if (datasize >= 5 && data[4] == 0) {
					mfifo++;
					if (!TT_SEQ_OK(ppktid, data[0]) && !initial1) {
						//printf("Flow error seq1: expecting %u, got %u\n", ppktid + 1, data[0]);
						logToFile("Flow error seq1: expecting %u, got %u\n", ppktid + 1, data[0]);
						flowerr++;
					}
					ppktid = data[0];
					initial1 = 0;
				}

				if (datasize >= 5 && data[4] == 1) {
					mhpfifo++;
					if (!TT_SEQ_OK(ppktid2, data[0]) && !initial2) {
						//printf("Flow error seq2: expecting %u, got %u\n", ppktid2 + 1, data[0]);
						logToFile("Flow error seq2: expecting %u, got %u\n", ppktid2 + 1, data[0]);
						flowerr++;
					}
					ppktid2 = data[0];
					initial2 = 0;
				}



			}

			bytesC += 6 + dmareq[i].RXBnDLC & 0xF;
			bytesD += dmareq[i].RXBnDLC & 0xF;
		}

		if (GetTickCount() - dwStart > 999) {

			printf("dev %d ch %d rx pkt: %d %d, b: %d, ds: %d\n", device_id, channel, blockC, GetTickCount() - dwStart, bytesC, bytesD);
			logToFile("dev %d ch %d rx pkt: %d %d, b: %d, ds: %d\n", device_id, channel, blockC, GetTickCount() - dwStart, bytesC, bytesD);
			if (mfifo > 0 || mhpfifo > 0) {
				printf("FIFO: %d, HPFIFO: %d\n", mfifo, mhpfifo);
				logToFile("FIFO: %d, HPFIFO: %d\n", mfifo, mhpfifo);
				mfifo = 0;
				mhpfifo = 0;
			}
			if (flowerr) {
				printf(" ERR_FLOW: %u", flowerr);
				logToFile(" ERR_FLOW: %u", flowerr);
			}
			if (blocksErr > 0) {
				printf(" ERR_CRC: %d", blocksErr);
				logToFile(" ERR_CRC: %d", blocksErr);
			}

			printf(" ");
			check_errors(&can, channel);
			printf("\n");

			blockC = 0;
			bytesC = 0;
			bytesD = 0;
			dwStart = GetTickCount();
		}

		if (blocks < DMA_STR_MAXSIZE) {
			Sleep(1);
		}
	}

	free(dmareq);

	return 0;
}