/**
* @file nmcantesttxfifo.cpp
*    FIFO, 1        FIFO
*/


#include "..\misc\misc.h"
#include "..\misc\tests.h"

void test_fifo_tx(HANDLE* can, int channel, int hpfifo) {
	CAN_FIFO_REQUEST fr;
	DWORD start, size, err;
	BOOL ioctlres;
	UINT8 pktid = 0;
	fr.channel = channel;

	start = GetTickCount();
	for (int i = 0;; i++) {
		switch (hpfifo) {
		case 0:
			fr.isHPFIFO = 0;
			break;
		case 1:
			fr.isHPFIFO = 1;
			break;
		case 2:
			fr.isHPFIFO = i % 2;
			break;
		}

		fr.size = build_seeded_data(fr.data, sizeof(fr.data), &(fr.sid), &(fr.eid), pktid, rand(), rand(), rand(), fr.isHPFIFO);
		logToFile("txfifo %3u s:x%3X e:x%8X l:%u", pktid, fr.sid, fr.eid, fr.size);
		pktid++;

		ioctlres = DeviceIoControl(*can, IOCTL_WRITE_DATA_FIFO, &fr, sizeof(CAN_FIFO_REQUEST), NULL, 0, &size, NULL);

		if (!ioctlres) {
			err = GetLastError();
			switch (err) {
			case 0xEA:
				Sleep(1);
				i--;
				pktid--;
				break;

			case 0x1:
				printf("Device channel not in FIFO mode or FIFO is unavailable\n");
				return;

			default:
				printf("Error x%X\n", err);
				return;
			}
		}

		if (GetTickCount() - start > 999) {
			printf("Speed: %d pkt/s %d\n", i / ((GetTickCount() - start) / 1000), GetTickCount() - start);
			check_errors(can, channel);
			i = 0;
			start = GetTickCount();
		}
	}

}

int main(int argc, char* argv[])
{
	CAN_device_list* devices;
	HANDLE can;
	DWORD ioctl = 0;
	int device_id, count;
	int channel, mode;

	std::cout << "CAN Serial Controller device test v." << TEST_VERSION << std::endl << COPYRIGHT << std::endl << "Built on " << __TIMESTAMP__ << std::endl << std::endl;

	std::cout << "Testing transfer in fifo mode" << std::endl;

	if (argc < 4) {
		std::cout << "usage: " << argv[0] << " <device_id> <channel: 0-1> <mode: 0 - normal FIFO | 1 - HPFIFO | 2 - use both> [dump]" << std::endl;
		return 1;
	}

	//setting timer precision, it will reset back to default after process finishes
	timeBeginPeriod(1);

	device_id = strtol(argv[1], NULL, 10);

	devices = CAN_enumerate();
	count = devices->count;

	if (count == 0) {
		std::cout << "No CAN devices found" << std::endl;
		return 1;
	}

	if (device_id >= count) {
		std::cout << "Invalid zero-based device number, devices available: " << count << std::endl;
		return 1;
	}

	std::cout << "found " << count << " devices" << std::endl;
	printf("Using %ws\n", devices->device[device_id].device_name);

	can = CAN_open_device(devices->device + device_id);

	CAN_enumerate_free(devices);

	if (can == INVALID_HANDLE_VALUE) {
		std::cout << "Bad handle" << std::endl;
		return 1;
	}

	channel = atoi(argv[2]);
	mode = atoi(argv[3]);

	if (channel != 0 && channel != 1) {
		std::cout << "Bad channel id" << std::endl;
		return 1;
	}

	if (mode != 0 && mode != 1 && mode != 2) {
		std::cout << "Bad FIFO mode" << std::endl;
		return 1;
	}

	if (argc > 4 && strcmp("dump", argv[4]) == 0) {
		gLogFile = openLogFile("can_txfifo");
	}

	printf("Using device %d, channel %d, mode %d\n", device_id, channel, mode);

	test_fifo_tx(&can, channel, mode);

	return 0;
}