/**
* @file nmcantesttxtg.cpp
*   
*/

#include "..\misc\misc.h"
#include "..\misc\tests.h"


void test_tx(HANDLE* can, int channel, int wait, int debug) {
	DWORD start, size, err;
	CAN_SEND_DATA sd;
	UINT8 pktid = 0, txbnctrl, prevbuf = 2;

	sd.nPriority = 0;
	sd.nBufNumber = 0;
	sd.timeout = 50;

	start = GetTickCount();
	for (int i = 0;; i++) {

		//use 2 buffers
		prevbuf = sd.nBufNumber;
		sd.nBufNumber = sd.nBufNumber ? 0 : 1;

		//wait transmit
		err = CAN_wait_tx(can, channel, sd.nBufNumber, 100);

		sd.nSize = build_seeded_data(sd.nData, sizeof(sd.nData), &(sd.SID), &(sd.EID), pktid, sd.nBufNumber, rand(), rand(), 0);
		if (debug) {
			printf("%3u s:x%3X e:x%8X l:%u\n", pktid, sd.SID, sd.EID, sd.nSize);
		}

		logToFile("tx %3u s:x%3X e:x%8X l:%u", pktid, sd.SID, sd.EID, sd.nSize);
		pktid++;

		//write txbuf, no autorts
		err = CAN_write_txbuf(can, channel, sd.nBufNumber, sd.nPriority, sd.timeout, sd.SID, sd.EID, sd.nData, sd.nSize, false);
		if (err) {
			printf("Error write txbuf x%X\n", err);
			return;
		}

		//check txbuf
		if (wait) {
			nmcan_wait_tx(can, channel, prevbuf);
		}

		//send data now
		CAN_send_data_now(can, channel, sd.nBufNumber);

		if (GetTickCount() - start > 999) {
			printf("Speed: %d pkt/s %d\n", i, GetTickCount() - start);
			i = 0;
			check_errors(can, channel);
			start = GetTickCount();
		}
	}

}

int main(int argc, char* argv[])
{
	CAN_device_list* devices;
	HANDLE can;
	DWORD ioctl = 0;
	int device_id, count, debug = 0;
	int channel, mode;

	std::cout << "CAN Serial Controller device test v." << TEST_VERSION << std::endl << COPYRIGHT << std::endl << "Built on " << __TIMESTAMP__ << std::endl << std::endl;

	std::cout << "Testing transfer in native mode" << std::endl;

	if (argc < 4) {
		std::cout << "usage: " << argv[0] << " <device_id> <channel: 0-1> <wait: 0|1> [debug|dump]" << std::endl;
		return 1;
	}

	//setting timer precision, it will reset back to default after process finishes
	timeBeginPeriod(1);

	device_id = strtol(argv[1], NULL, 10);

	devices = CAN_enumerate();
	count = devices->count;

	if (count == 0) {
		std::cout << "No CAN devices found" << std::endl;
		return 1;
	}

	if (device_id >= count) {
		std::cout << "Invalid zero-based device number, devices available: " << count << std::endl;
		return 1;
	}

	std::cout << "found " << count << " devices" << std::endl;
	printf("Using %ws\n", devices->device[device_id].device_name);

	can = CAN_open_device(devices->device + device_id);

	CAN_enumerate_free(devices);

	if (can == INVALID_HANDLE_VALUE) {
		std::cout << "Bad handle" << std::endl;
		return 1;
	}

	channel = atoi(argv[2]);
	mode = atoi(argv[3]);

	if (channel != 0 && channel != 1) {
		std::cout << "Bad channel id" << std::endl;
		return 1;
	}

	if (mode != 0 && mode != 1 && mode != 2) {
		std::cout << "Bad FIFO mode" << std::endl;
		return 1;
	}

	if (argc > 4 && strcmp("debug", argv[4]) == 0) {
		debug = 1;
		std::cout << "Debug mode" << std::endl;
	}

	if (argc > 4 && strcmp("dump", argv[4]) == 0) {
		gLogFile = openLogFile("can_txq");
	}

	printf("Using device %d, channel %d, wait %d\n", device_id, channel, mode);

	test_tx(&can, channel, mode, debug);

	return 0;
}